import { FederationServiceTypes } from '@cross-lab-project/api-client';
import { LitElement, html } from 'lit';
import { customElement, query, state } from 'lit/decorators.js';
import { apiClient } from '../../../globals';
import { Editor } from '../common';

@customElement('apitool-institution-creator')
export class InstitutionCreator extends LitElement {
    @state()
    institution: FederationServiceTypes.Institution<'request'> = {
        api: '',
        apiToken: '',
        homepage: '',
        name: '',
    };

    @query('#input-api')
    inputApi!: HTMLInputElement;

    @query('#input-api-token')
    inputApiToken!: HTMLInputElement;

    @query('#input-homepage')
    inputHomepage!: HTMLInputElement;

    @query('#input-name')
    inputName!: HTMLInputElement;

    @query('apitool-editor')
    editor!: Editor;

    protected createRenderRoot(): Element | ShadowRoot {
        return this;
    }

    protected render(): unknown {
        return html`<apitool-editor
            .type=${'creator'}
            @create-resource=${this.createInstitution}
            @cancel=${this.cancel}
        >
            <div class="flex flex-col">
                <p>Name:</p>
                <input
                    id="input-name"
                    type="text"
                    class="w-full p-2 resize-none border rounded-lg"
                    @input=${(event: InputEvent) => {
                        this.institution.name = (
                            event.target as HTMLInputElement
                        ).value;
                        this.editor.messageField.removeAllSuccessMessages();
                    }}
                />
            </div>
            <div class="flex flex-col">
                <p>Homepage:</p>
                <input
                    id="input-homepage"
                    type="text"
                    class="w-full p-2 resize-none border rounded-lg"
                    @input=${(event: InputEvent) => {
                        this.institution.homepage = (
                            event.target as HTMLInputElement
                        ).value;
                        this.editor.messageField.removeAllSuccessMessages();
                    }}
                />
            </div>
            <div class="flex flex-col">
                <p>API URL:</p>
                <input
                    id="input-api"
                    type="text"
                    class="w-full p-2 resize-none border rounded-lg"
                    @input=${(event: InputEvent) => {
                        this.institution.api = (
                            event.target as HTMLInputElement
                        ).value;
                        this.editor.messageField.removeAllSuccessMessages();
                    }}
                />
            </div>
            <div class="flex flex-col">
                <p>API Token:</p>
                <input
                    id="input-api-token"
                    type="text"
                    class="w-full p-2 resize-none border rounded-lg"
                    @input=${(event: InputEvent) => {
                        this.institution.apiToken = (
                            event.target as HTMLInputElement
                        ).value;
                        this.editor.messageField.removeAllSuccessMessages();
                    }}
                />
            </div>
        </apitool-editor>`;
    }

    private async createInstitution() {
        console.log('trying to create institution:', this.institution);
        try {
            const createdInstitution = await apiClient.createInstitution(
                this.institution
            );

            const newUrl =
                '/institutions/' + createdInstitution.url.split('/').at(-1);

            const event = new CustomEvent<string>('update-view', {
                detail: newUrl,
                bubbles: true,
            });

            this.dispatchEvent(event);

            this.editor.messageField.addMessage(
                'success',
                'Institution created successfully!'
            );
            console.log(
                'institution created successfully:',
                createdInstitution
            );
        } catch (error) {
            this.editor.messageField.addMessage(
                'error',
                JSON.stringify(error, null, 4)
            );
        }
    }

    private cancel() {
        const event = new CustomEvent<string>('update-view', {
            detail: '/institutions',
            bubbles: true,
        });

        this.dispatchEvent(event);
    }
}
