import { LitElement, html } from 'lit';
import { customElement, query } from 'lit/decorators.js';

export type InstitutionFilterOptions = {
    url: string;
    name: string;
    homepage: string;
    api: string;
};

@customElement('apitool-institution-list-view-filter')
export class InstitutionListViewFilter extends LitElement {
    @query('#url-filter')
    urlFilter!: HTMLInputElement;

    @query('#name-filter')
    nameFilter!: HTMLInputElement;

    @query('#homepage-filter')
    homepageFilter!: HTMLInputElement;

    @query('#api-filter')
    apiFilter!: HTMLInputElement;

    protected createRenderRoot(): Element | ShadowRoot {
        return this;
    }

    protected render(): unknown {
        return html`<apitool-collapsable-element
            .title=${'Filters'}
            class="bg-slate-300 rounded-lg p-2 flex flex-col border-2 border-black ml-[env(safe-area-inset-left)] mr-[env(safe-area-inset-right)]"
        >
            <div class="w-full flex flex-col gap-1">
                <input
                    id="url-filter"
                    type="text"
                    placeholder="URL"
                    class="p-2 rounded-lg mb-1 border-2 border-black"
                    @input=${this.updateFilters}
                />
                <input
                    id="name-filter"
                    type="text"
                    placeholder="Name"
                    class="p-2 rounded-lg mb-1 border-2 border-black"
                    @input=${this.updateFilters}
                />
                <input
                    id="homepage-filter"
                    type="text"
                    placeholder="Homepage"
                    class="p-2 rounded-lg mb-1 border-2 border-black"
                    @input=${this.updateFilters}
                />
                <input
                    id="api-filter"
                    type="text"
                    placeholder="API URL"
                    class="p-2 rounded-lg mb-1 border-2 border-black"
                    @input=${() => this.updateFilters()}
                />
            </div>
        </apitool-collapsable-element> `;
    }

    private updateFilters() {
        const event = new CustomEvent<InstitutionFilterOptions>(
            'filters-updated',
            {
                detail: {
                    url: this.urlFilter.value,
                    name: this.nameFilter.value,
                    homepage: this.homepageFilter.value,
                    api: this.apiFilter.value,
                },
            }
        );

        this.dispatchEvent(event);
    }
}
