import { FederationServiceTypes } from '@cross-lab-project/api-client';
import { LitElement, html } from 'lit';
import { customElement, property, state } from 'lit/decorators.js';

@customElement('apitool-institution-list-view-item')
export class InstitutionListViewItem extends LitElement {
    @property({ type: Object })
    institution!: FederationServiceTypes.Institution<'response'>;

    @state()
    isOpen: boolean = false;

    protected createRenderRoot(): Element | ShadowRoot {
        return this;
    }

    protected render(): unknown {
        return html`<apitool-collapsable-element
            .title=${this.institution.name}
            class="flex p-2 border-2 border-black rounded-lg bg-slate-300 ml-[env(safe-area-inset-left)] mr-[env(safe-area-inset-right)]"
        >
            <div class="bg-slate-100 rounded-lg w-full p-2">
                ${this.renderInformation()}
                <div class="mt-2 flex flex-row gap-2">
                    <button
                        @click=${() => this.editInstitution()}
                        class="w-full justify-center bg-slate-600 text-gray-50 p-2 rounded-lg ml-auto flex hover:cursor-pointer hover:bg-slate-700 active:bg-slate-800"
                    >
                        Edit
                    </button>
                    <button
                        @click=${() => this.deleteInstitution()}
                        class="w-full justify-center bg-red-600 text-gray-50 p-2 rounded-lg ml-auto flex hover:cursor-pointer hover:bg-red-700 active:bg-red-800"
                    >
                        Delete
                    </button>
                </div>
            </div>
        </apitool-collapsable-element>`;
    }

    private renderInformation() {
        return html`<div class="flex flex-col">
            <div class="flex">
                <p class="w-28 flex-shrink-0">URL:</p>
                <p class="whitespace-nowrap overflow-hidden text-ellipsis">
                    ${this.institution.url}
                </p>
            </div>
            <div class="flex">
                <p class="w-28 flex-shrink-0">Name:</p>
                <p class="whitespace-nowrap overflow-hidden text-ellipsis">
                    ${this.institution.name}
                </p>
            </div>
            <div class="flex">
                <p class="w-28 flex-shrink-0">Homepage:</p>
                <p class="whitespace-nowrap overflow-hidden text-ellipsis">
                    ${this.institution.homepage}
                </p>
            </div>
            <div class="flex">
                <p class="w-28 flex-shrink-0">API URL:</p>
                <p class="whitespace-nowrap overflow-hidden text-ellipsis">
                    ${this.institution.api}
                </p>
            </div>
        </div>`;
    }

    private async editInstitution() {
        const event = new CustomEvent<string>('update-view', {
            detail: '/institutions/' + this.institution.url.split('/').at(-1),
            bubbles: true,
        });

        this.dispatchEvent(event);
    }

    private async deleteInstitution() {
        const event = new CustomEvent<string>('delete-institution', {
            detail: this.institution.url,
            bubbles: true,
        });

        this.dispatchEvent(event);
    }
}
